@@ -21,39 +21,63 @@ const notebook = require('./generate-notebook-renderer');
 const snippets = require('./generate-snippets');
 const webupdate = require('./generate-web-update');
 
+const extensionGenerators = [
+    commandts, commandjs, colortheme, language, snippets, keymap, extensionpack, localization,
+    commandweb, notebook, webupdate
+]
+
 module.exports = class extends Generator {
 
     constructor(args, opts) {
         super(args, opts);
-        this.option('extensionType', { type: String });
-        this.option('extensionName', { type: String });
-        this.option('extensionDescription', { type: String });
-        this.option('extensionDisplayName', { type: String });
+        this.description = 'Generates a Visual Studio Code extension ready for development.';
 
-        this.option('extensionParam', { type: String });
-        this.option('extensionParam2', { type: String });
+        this.argument('destination', { type: String, required: false, description: `\n    The folder to create the extension in, absolute or relative to the current working directory.\n    Use '.' for the current folder. If not provided, defaults to a folder with the extension display name.\n  ` })
+
+        this.option('insiders', { type: Boolean, alias: 'i', description: 'Show the insiders options for the generator' });
+        this.option('quick', { type: Boolean, alias: 'q', description: 'Quick mode, skip all optional prompts and use defaults' });
+        this.option('open', { type: Boolean, alias: 'o', description: 'Open the generated extension in Visual Studio Code' });
+        this.option('openInInsiders', { type: Boolean, alias: 'O', description: 'Open the generated extension in Visual Studio Code Insiders' });
+
+        this.option('extensionType', { type: String, alias: 't', description: extensionGenerators.slice(0, 6).map(e => e.aliases[0]).join(', ') + '...' });
+        this.option('extensionDisplayName', { type: String, alias: 'n', description: 'Display name of the extension' });
+        this.option('extensionId', { type: String, description: 'Id of the extension' });
+        this.option('extensionDescription', { type: String, description: 'Description of the extension' });
+
+        this.option('pkgManager', { type: String, description: `'npm' or 'yarn'` });
+        this.option('webpack', { type: Boolean, description: `Bundle the extension with webpack` });
+        this.option('gitInit', { type: Boolean, description: `Initialize a git repo` });
+
+        this.option('snippetFolder', { type: String, description: `Snippet folder location` });
+        this.option('snippetLanguage', { type: String, description: `Snippet language` });
 
         this.extensionConfig = Object.create(null);
         this.extensionConfig.installDependencies = false;
+        this.extensionConfig.insiders = false;
 
         this.extensionGenerator = undefined;
 
         this.abort = false;
-
-        this.insiders = false;
     }
 
     async initializing() {
-        const cliArgs = this.options['_'];
-        this.insiders = Array.isArray(cliArgs) && cliArgs.indexOf('insiders') !== -1;
+        if (this.options['insiders']) {
+            this.extensionConfig.insiders = true;
+        }
 
         // Welcome
-        if (!this.insiders) {
+        if (!this.extensionConfig.insiders) {
             this.log(yosay('Welcome to the Visual Studio Code Extension generator!'));
         } else {
             this.log(yosay('Welcome to the Visual Studio Code Insiders Extension generator!'));
         }
 
+        const destination = this.options['destination'];
+        if (destination) {
+            const folderPath = path.resolve(this.destinationPath(), destination);
+            this.destinationRoot(folderPath);
+        }
+
         // evaluateEngineVersion
         const dependencyVersions = await env.getDependencyVersions();
         this.extensionConfig.dependencyVersions = dependencyVersions;
@@ -68,23 +92,26 @@ module.exports = class extends Generator {
     }
 
     async prompting() {
-        const extensionGenerators = [
-            commandts, commandjs, colortheme, language, snippets, keymap, extensionpack, localization,
-            commandweb, notebook, webupdate
-        ]
 
         // Ask for extension type
         const extensionType = this.options['extensionType'];
         if (extensionType) {
             const extensionTypeId = 'ext-' + extensionType;
-            if (extensionGenerators.find(g => g.id === extensionTypeId)) {
-                this.extensionConfig.type = extensionTypeId;
+            const extensionGenerator = extensionGenerators.find(g => g.aliases.indexOf(extensionType) !== -1);
+            if (extensionGenerator) {
+                this.extensionConfig.type = extensionGenerator.id;
             } else {
-                this.log("Invalid extension type: " + extensionType + '\nPossible types are: ' + extensionGenerators.map(g => g.id.substr(4)).join(', '));
+                this.log("Invalid extension type: " + extensionType + '\nPossible types are: ' + extensionGenerators.map(g => g.aliases.join(', ')).join(', '));
                 this.abort = true;
             }
         } else {
-            const choices = extensionGenerators.filter(g => !!g.insiders === this.insiders).map(g => ({ name: g.name, value: g.id }));
+            const choices = [];
+            for (const g of extensionGenerators) {
+                const name = this.extensionConfig.insiders ? g.insidersName : g.name;
+                if (name) {
+                    choices.push({ name, value: g.id })
+                }
+            }
             this.extensionConfig.type = (await this.prompt({
                 type: 'list',
                 name: 'type',
@@ -108,6 +135,14 @@ module.exports = class extends Generator {
         if (this.abort) {
             return;
         }
+        if (!this.options['destination'] && !this.extensionGenerator.update) {
+            this.destinationRoot(this.destinationPath(this.extensionConfig.name))
+        }
+        this.env.cwd = this.destinationPath();
+
+        this.log();
+        this.log(`Writing in ${this.destinationPath()}...`);
+
         this.sourceRoot(path.join(__dirname, './templates/' + this.extensionConfig.type));
 
         return this.extensionGenerator.writing(this, this.extensionConfig);
@@ -116,17 +151,13 @@ module.exports = class extends Generator {
     // Installation
     install() {
         if (this.abort) {
+            this.env.options.skipInstall = true;
             return;
         }
-        if (!this.extensionGenerator.update) {
-            process.chdir(this.extensionConfig.name);
-        }
         if (this.extensionConfig.installDependencies) {
-            this.installDependencies({
-                yarn: this.extensionConfig.pkgManager === 'yarn',
-                npm: this.extensionConfig.pkgManager === 'npm',
-                bower: false
-            });
+            this.env.options.nodePackageManager = this.extensionConfig.pkgManager;
+        } else {
+            this.env.options.skipInstall = true;
         }
     }
 
@@ -142,7 +173,7 @@ module.exports = class extends Generator {
             this.log('');
             this.log('To start editing with Visual Studio Code, use the following commands:');
             this.log('');
-            if (!this.insiders) {
+            if (!this.extensionConfig.insiders) {
                 this.log('     code .');
             } else {
                 this.log('     code-insiders .');
@@ -157,29 +188,27 @@ module.exports = class extends Generator {
             this.spawnCommand('git', ['init', '--quiet']);
         }
 
+        if (this.extensionConfig.proposedAPI) {
+            this.spawnCommand(this.extensionConfig.pkgManager, ['run', 'update-proposed-api']);
+        }
         this.log('');
 
-        if (!this.insiders) {
-            this.log('Your extension ' + this.extensionConfig.name + ' has been created!');
-            this.log('');
+        this.log('Your extension ' + this.extensionConfig.name + ' has been created!');
+        this.log('');
+
+        if (!this.extensionConfig.insiders && !this.options['open'] && !this.options['openInInsiders'] && !this.options['quick']) {
+            const cdLocation = this.options['destination'] || this.extensionConfig.name;
+
             this.log('To start editing with Visual Studio Code, use the following commands:');
             this.log('');
-            this.log('     cd ' + this.extensionConfig.name);
-            this.log('     code .');
-            this.log('');
-            this.log('Open vsc-extension-quickstart.md inside the new extension for further instructions');
-            this.log('on how to modify, test and publish your extension.');
-        } else {
-            this.log('Your extension ' + this.extensionConfig.name + ' has been created!');
-            this.log('');
-            this.log('To start editing with Visual Studio Code, use the following commands:');
-            this.log('');
-            this.log('     cd ' + this.extensionConfig.name);
+            if (cdLocation !== '.') {
+                this.log('     cd ' + cdLocation);
+            }
             this.log('     code-insiders .');
             this.log('');
-            this.log('Open vsc-extension-quickstart.md inside the new extension for further instructions');
-            this.log('on how to modify and test your extension.');
         }
+        this.log('Open vsc-extension-quickstart.md inside the new extension for further instructions');
+        this.log('on how to modify, test and publish your extension.');
         this.log('');
 
         if (this.extensionGenerator.endMessage) {
@@ -188,5 +217,13 @@ module.exports = class extends Generator {
 
         this.log('For more information, also visit http://code.visualstudio.com and follow us @code.');
         this.log('\r\n');
+
+        if (this.options['open']) {
+            this.log(`Opening ${this.destinationPath()} in Visual Studio Code...`);
+            this.spawnCommand('code', [this.destinationPath()]);
+        } else if (this.extensionConfig.insiders || this.options['openInInsiders'] || this.options['quick']) {
+            this.log(`Opening ${this.destinationPath()} with Visual Studio Code Insiders...`);
+            this.spawnCommand('code-insiders', [this.destinationPath()]);
+        }
     }
 }
