@@ -4,10 +4,11 @@ var path = require('path');
 var globby = require('globby');
 var multimatch = require('multimatch');
 var util = require('../util');
+var {setDeletedFileState} = require('../state');
 
 function deleteFile(path, store) {
   var file = store.get(path);
-  file.state = 'deleted';
+  setDeletedFileState(file);
   file.contents = null;
   store.add(file);
 }
